#!/usr/bin/env ruby

 ###############################################################################
 #  Copyright 2008 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

Dir.chdir(File.dirname(__FILE__))	# So that this file can be run from anywhere

MOUSE_BUTTON_1, MOUSE_BUTTON_2, MOUSE_BUTTON_3 = (1..3).to_a

# Provide nice error message for common missing packages (libnotify-bin is good too)
begin
	require 'gtk2'
	require 'libglade2'
	require 'gconf2'
	require 'sqlite3'
rescue LoadError => e
	puts 'Some required files are missing:'
	puts '-> Ubuntu users type: sudo apt-get install libgtk2-ruby libsqlite3-ruby libglade2-ruby libgconf2-ruby'
	puts
	puts 'The error was:'
	puts e
	exit
end

include Gtk

require 'helpers'
require 'constants'
require 'history'
require 'language'
require 'dictionary_window'
require 'preferences_window'
#require 'status_icon_menu'

class Application
	def initialize
		$dictionary_window = DictionaryWindow.new
		$preferences_window = PreferencesWindow.new

		#	Tray/status icon menu
		@glade = GladeXML.new(GLADE_FILE_NAME, 'tray_icon_menu') { | handler_name | method(handler_name) }
		$status_icon_menu = @glade.get_widget('tray_icon_menu')
		#$status_icon_menu = StatusIconMenu.new

		#	Create tray/status icon
		@status_icon = Gtk::StatusIcon.new
		@status_icon.set_pixbuf($icon_tray)
		@status_icon.signal_connect('activate')	{ $dictionary_window.toggle_visibility }
		@status_icon.signal_connect('popup-menu')	{ |icon, button, time| $status_icon_menu.popup(nil, nil, button, time) { @status_icon.position_menu($status_icon_menu) } }

		# Clipboard watcher
		@last_clipboard_text = ''
		Gtk.timeout_add(CLIPBOARD_WATCH_TIMEOUT) { $clipboard.request_text { |clipboard, text| on_receive_clipboard_text(text) } if $preferences_window.watch_clipboard_checkbutton.active? }
	end

	def on_receive_clipboard_text(text)
		#	Because we have to poll, it only makes sense to react to changes in selection text
		return if text == @last_clipboard_text

		@last_clipboard_text = text			# do this before checking for empty text, so we "reset" when the user deselects text
		return if (text.nil? || text.empty?)

		# don't respond to selections in our own window(s)
		return if ($dictionary_window.has_selection || $preferences_window.has_selection)

		# be friendlier to mouse-cowboys (remove junk characters in selection)
		text = $foreign_language.class.clean_string(text)

		#	We can ignore garbage
		return unless $foreign_language.class.resembles_word(text)

		# Search for word
		text = $foreign_language.class.downcase(text)
		if $dictionary_window.search(text)	# something found?
			timeout = 10000
			notify(:summary => text, :body => $dictionary_window.notification_summary, :timeout => timeout, :urgency => 'low') if $preferences_window.show_notifications_checkbutton.active?
			$dictionary_window.urgency_hint = true
		end
	end

	def on_show_activate
		$dictionary_window.toggle_visibility
	end

	def on_preferences_activate
		$preferences_window.present
	end

	def on_quit_activate
		Gtk.main_quit
	end

	def on_clipboard_watch_timeout
		$clipboard.request_text { | clipboard, text | on_receive_clipboard_text(text) } if $preferences_window.watch_clipboard_checkbutton.active?
		return true
	end
end

#
# main :)
#
Gtk.init

$clipboard = Gtk::Clipboard.get('PRIMARY')

#	TODO:	switch to ActiveRecords
$db = SQLite3::Database.new(DATABASE_FILE_NAME)
$db.type_translation = true
$db.results_as_hash = true

#$icon = Gdk::Pixbuf.new(ICON_PATH, 32,32)
$icon_tray = Gdk::Pixbuf.new(ICON_PATH, 22,22)

# Init languages TODO:	some way to make language choice dynamic
require 'language_english'
$native_language = LanguageEnglish.new

require 'language_spanish'
$foreign_language = LanguageSpanish.new

$app = Application.new
Gtk.main
