 ###############################################################################
 #  Copyright 2008 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

require 'glade_window'
require 'helpers'
require 'constants'
require 'textview_tags'

class DictionaryWindow < GladeWindow
	attr_reader :notification_summary

	def initialize
		super('dictionary_window')

#		after_filter(:update_history_buttons, :only => ['on_back_button_clicked', 'on_forward_button_clicked'])
#		after_filter(:update_search_box,  :only => ['on_back_button_clicked', 'on_forward_button_clicked'])

		@history = History.new
		@window.icon = $icon
		@buffer = @textview.buffer	# for easier access

		$textview_tags.each { |name, options| @buffer.create_tag(name,options) }

		@preferences_button.signal_connect('clicked') { $preferences_window.present }
		@notification_summary = ''
	end

public

	###################################################################
	# Status
	###################################################################
	def has_selection
		return @search_entry.has_selection || @buffer.has_selection
	end

	###################################################################
	# Actions
	###################################################################
	def search(search_words)
		puts "search_words: " + search_words
		if load_uri(search_words) > 0
			@statusbar_label.markup = ""
			@history.add(search_words)
			@search_entry.set_text(search_words)
			update_history_buttons			# TODO: move to hook?
			return true
		else
			@statusbar_label.markup = "<i>'" + search_words.limit(STATUSBAR_CHARACTER_LIMIT) + "' not found</i>"
			return false
		end
	end

	def load_uri(search)
		foreign_results = $foreign_language.search(search)
		return 0 if foreign_results.size == 0
		@textview.clear 		# AFTER checking for 0 results (don't clear if there's nothing new to show)
		$foreign_language.render_results_to_textbuffer(foreign_results, @buffer)
		@notification_summary = $foreign_language.render_results_to_notification(foreign_results)
		return foreign_results.size
	end

	def toggle_visibility
		unless active?													# becoming visible?
			@search_entry.select_all.grab_focus		# make it usable without tabbing
		end
		super
	end

private

	###################################################################
	# GUI Updaters
	###################################################################
	def update_history_buttons
		@back_button.sensitive = @history.can_go_back
		@forward_button.sensitive = @history.can_go_forward
	end

	def update_search_box
		@search_entry.set_text(@history.current).select_all
	end

	###################################################################
	# Signal Handlers
	###################################################################
	def on_search_entry_activate
		return if @search_entry.text.strip.empty?
		search(@search_entry.text)
		@search_entry.select_all
	end

	def on_back_button_clicked
		@history.back
		load_uri(@history.current)
		update_history_buttons
		update_search_box
	end

	def on_forward_button_clicked
		@history.forward
		load_uri(@history.current)
		update_history_buttons
		update_search_box
	end

	def on_key_press_event(obj, event)
		case event.keyval
			when Gdk::Keyval::GDK_Page_Down then @textview.move_viewport(Gtk::SCROLL_PAGES, 1)
			when Gdk::Keyval::GDK_Page_Up then @textview.move_viewport(Gtk::SCROLL_PAGES, -1) 
			when Gdk::Keyval::GDK_Down then @textview.move_viewport(Gtk::SCROLL_STEPS, 1)
			when Gdk::Keyval::GDK_Up then @textview.move_viewport(Gtk::SCROLL_STEPS, -1)
			else return super(obj, event)
		end
		return true	# handled
	end
end
