 ###############################################################################
 #  Copyright 2008 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

require 'language'

class LanguageSpanish < Language

	VERB_PERSON_NONE = 0
	VERB_PERSON_YO = 1
	VERB_PERSON_TU = 2
	VERB_PERSON_EL = 3
	VERB_PERSON_NOSOTROS = 4
	VERB_PERSON_VOSOTROS = 5
	VERB_PERSON_ELLOS = 6

	@@verb_person_names = ['', 'yo', 'tú', 'él/ella/Ud', 'nosotros', 'vosotros', 'ellos/ellas/Uds' ]
	@@verb_person_show = [true, true, true, true, true, false, true]

	VERB_TENSE_NONE = 0
	VERB_TENSE_INFINITIVE = 0

	VERB_TENSE_PRESENT = 1
	VERB_TENSE_PRETERITE = 2
	VERB_TENSE_FUTURE = 3
	VERB_TENSE_IMPERFECT = 4
	VERB_TENSE_CONDITIONAL = 5
	VERB_TENSE_SUBJUNCTIVE_PRESENT = 6
	VERB_TENSE_SUBJUNCTIVE_IMPERFECT = 7
	VERB_TENSE_SUBJUNCTIVE_FUTURE = 8
	VERB_TENSE_IMPERATIVE = 9
	VERB_TENSE_IMPERATIVE_NEGATIVE = 10
	VERB_TENSE_GERUND = 11
	VERB_TENSE_PAST_PARTICIPLE = 12

	@@tense_names = ['infinitive', 'present', 'past preterite', 'future', 'past imperfect', 'future conditional', 'present subjunctive', 'past subjunctive', 'future subjunctive', 'imperative', 'imperative (negative)', 'progressive', 'past participle']
	# the correct names:
	# %@@tense_names = {0 => '', 1 => 'present', 2 => 'preterite', 3 => 'future', 4 => 'imperfect', 5 => 'future conditional', 6 => 'present subjunctive', 7 => 'imperfect subjunctive', 8 => 'future subjunctive', 9 => 'imperative', 10 => 'imperative (negative)', 11 => 'gerund', 12 => 'past participle'}

	# TODO: move to user Preferences?
	@@tense_show = [false, true, true, true, true, true, true, true, true, true, true, true, true]
#	@@tense_show = [false, true, true, true, true, true, true, true, true, true, true, true, true]

	# some tenses add on to the infinitive (eg "hablar+é") instead of using the verb root ("habl+o")
	@@tense_uses_infinitive = [false, false, false, true, false, true, false, false, false, false, false, false, false]

	# here are all the verb endings which either get added to the root or the infinitive (based on @@tense_uses_infinitive)
	@@regular_conjugations = {
		'ar' => [[],['', 'o', 'as', 'a', 'amos', 'áis', 'an'],['', 'é', 'aste', 'ó', 'amos', 'asteis', 'aron'],['', 'é', 'ás', 'á', 'emos', 'éis', 'án'],['', 'aba', 'abas', 'aba', 'ábamos', 'abais', 'aban'],['', 'ía', 'ías', 'ía', 'íamos', 'íais', 'ían'],['', 'e', 'es', 'e', 'emos', 'éis', 'en'],['', 'ara', 'aras', 'ara', 'áramos', 'arais', 'aran'],['', 'are', 'ares', 'are', 'áremos', 'areis', 'aren'],['', '', 'a', 'e', 'emos', 'ad', 'en'],['', '', 'es', 'e', 'emos', 'éis', 'en'],['ando','','','','','',''],['ado','','','','','','']],
		'er' => [[],['', 'o', 'es', 'e', 'emos', 'éis', 'en'],['', 'í', 'iste', 'ió', 'imos', 'isteis', 'ieron'],['', 'é', 'ás', 'á', 'emos', 'éis', 'án'],['', 'ía', 'ías', 'ía', 'íamos', 'íais', 'ían'],['', 'ía', 'ías', 'ía', 'íamos', 'íais', 'ían'],['', 'a', 'as', 'a', 'amos', 'áis', 'an'],['', 'iera', 'ieras', 'iera', 'iéramos', 'ierais', 'ieran'],['', 'iere', 'ieres', 'iere', 'iéremos', 'iereis', 'ieren'],['', '', 'e', 'a', 'amos', 'ed', 'an'],['', '', 'as', 'a', 'amos', 'áis', 'an'],['iendo','','','','','',''],['ido','','','','','','']],
		'ir' => [[],['', 'o', 'es', 'e', 'imos', 'ís', 'en'],['', 'í', 'iste', 'ió', 'imos', 'isteis', 'ieron'],['', 'é', 'ás', 'á', 'emos', 'éis', 'án'],['', 'ía', 'ías', 'ía', 'íamos', 'íais', 'ían'],['', 'ía', 'ías', 'ía', 'íamos', 'íais', 'ían'],['', 'a', 'as', 'a', 'amos', 'áis', 'an'],['', 'iera', 'ieras', 'iera', 'iéramos', 'ierais', 'ieran'],['', 'iere', 'ieres', 'iere', 'iéremos', 'iereis', 'ieren'],['', '', 'e', 'a', 'amos', 'id', 'an'],['', '', 'as', 'a', 'amos', 'áis', 'an'],['iendo','','','','','',''],['ido','','','','','','']]
	}

public

	###################################################################
	# Language Utilities
	###################################################################
	def self.tense_name(id)
		@@tense_names[id]
	end

	def self.person_name(id)
		@@verb_person_names[id]
	end

	def self.clean_string(str)
		return str.gsub(/[\,\.\!\?\:\'\"\[\]\(\)\{\}\/]/, ' ').strip
	end

	def self.resembles_word(str)
		return (str =~ /\A[a-zA-ZÁÉÍÓÚÑáéíóúñ]+\Z/u) != nil
	end

	def self.downcase(str)
		return str.tr("ÁÉÍÓÚÑ", "áéíóúñ").downcase
	end

	def self.strip_accents(str)	# better way to do this?
		return str.gsub(/á/u, 'a').gsub(/é/u, 'e').gsub(/í/u, 'i').gsub(/ó/u, 'o').gsub(/ú/u, 'u').gsub(/ñ/u, 'n')
	end

	###################################################################
	# Search
	###################################################################

	def get_word_by_id(word_id)
		$db.execute('SELECT * FROM spanish_word WHERE id=?', word_id).first
	end

	def search(search_string)
		results = Array.new

		#puts "searching: #{search_string}"
		# real words (including verbs (infinitive form), nouns, pronouns, etc.)
		($db.execute('SELECT * FROM spanish_word WHERE word LIKE ?', search_string) || []).each { |row|
			#puts "adding whole word result: #{row['word']}"
			results << {:word_type => row['type'], :verb_tense => VERB_TENSE_NONE, :verb_person => VERB_PERSON_NONE, :word_id => row['id'], :search => row['word']}
		}

		# irregular verb conjugations
		sql = 'SELECT spanish_verb_conjugation.*, spanish_word.word FROM spanish_verb_conjugation LEFT JOIN spanish_word ON (spanish_verb_conjugation.word_id=spanish_word.id) WHERE conjugation LIKE ?'
		($db.execute(sql, search_string) || []).each { |row|
			#puts "adding irregular result: #{row}"

			results << {:word_type => WORD_TYPE_VERB, :verb_tense => row['tense'], :verb_person => row['person'], :word_id => row['word_id'], :search => search_string}
		}

		# regular verb conjugations (not stored in DB)
		unconjugate(search_string) { | infinitive, tense, person |
			# it's a regular conjugation IF the infinitive exist AND no irregular for this verb/person/tense exists
			infinitive = $db.get_first_row("SELECT * FROM spanish_word WHERE word=? AND type=?", infinitive, WORD_TYPE_VERB)
			next if infinitive == nil	# not a verb
			irregular = $db.get_first_row("SELECT * FROM spanish_verb_conjugation WHERE word_id=? AND tense=? AND person=?", infinitive['id'].to_i, tense, person)
			next if irregular != nil	# oops-- it's irregular in this tense/person, so no match
			#puts 'adding regular conjugation result'
			results << {:word_type => WORD_TYPE_VERB, :verb_tense => tense, :verb_person => person, :word_id => infinitive['id'].to_i, :search => search_string}
		}
		return results
	end

	###################################################################
	# conjugate() - change an infinitive "hablar" to a specific tense/person conjugation like "hablo" (present tense, first person) using lookup tables
	###################################################################
	def self.conjugate(infinitive, tense, person)
		#infinitive.gsub!(/se\Z/, '')	# better way?
		infinitive = infinitive[0, infinitive.length-2] if infinitive[infinitive.length-2, 2] == 'se'	# remove 'se' if present
		infinitive = strip_accents(infinitive)
		stem = infinitive[0, infinitive.length-2]
		ar_er_ir = infinitive[infinitive.length-2, 2]
		if(@@tense_uses_infinitive[tense])	# does it add on to the infinitive?
			return infinitive + @@regular_conjugations[ar_er_ir][tense][person] if @@regular_conjugations[ar_er_ir][tense][person] != ''
			return nil
		else
			return stem + @@regular_conjugations[ar_er_ir][tense][person] if @@regular_conjugations[ar_er_ir][tense][person] != ''
			return nil
		end
	end

	###################################################################
	# unconjugate() - take what MIGHT be a conjugated verb and 'yield' possible infinitives
	# most returned values are JUNK and so caller must check that:
	#  1) the returned infinitive exists,
	#  2) the verb isn't irregular in that tense/person
	###################################################################
	def unconjugate(conjugation)	# yields each possible | infinitive, tense, person |
		@@regular_conjugations.each_key { | ar_er_ir |
			@@regular_conjugations[ar_er_ir].each_index { | tense |
				@@regular_conjugations[ar_er_ir][tense].each_index { | person |
					tail = @@regular_conjugations[ar_er_ir][tense][person]				# one possible tail for the conjugated verb
					next if tail.length == 0 || tail.length >= conjugation.length			# can't possibly match

					# does it have this tail at the end?
					next if tail != conjugation[conjugation.length-tail.length, conjugation.length]

					# yield results
					before_tail = conjugation[0, conjugation.length-tail.length]
					if @@tense_uses_infinitive[tense]
						yield before_tail, tense, person							# bit before the tail is the infinitive
					else
						yield before_tail + ar_er_ir, tense, person		# bit before the tail is the infinitive minus -ar/-er/-ir
					end
				}
			}
		}
	end

	###################################################################
	# render_results_to_textbuffer() - take an array of results (as created
	#   by search() above) and output to a GTK TextBuffer
	###################################################################
	def render_results_to_textbuffer(results, buffer)
		last_word_id = nil

		results.each { |result|
			# We can have many results with the same base word
			# (we expect them to be sorted by word or word_id)
			if last_word_id != result[:word_id]
				#
				word = $db.get_first_row("SELECT * FROM spanish_word WHERE id=?", result[:word_id])
				definitions = $db.execute("SELECT english_word.* FROM spanish_english LEFT JOIN english_word ON (english_word.id=spanish_english.english_word_id) WHERE spanish_english.spanish_word_id=?", result[:word_id])

				# blank line *between* different words (not before the first)
				buffer.insert(buffer.end_iter, "\n") unless last_word_id.nil?

				# show base word ("hablar")
				buffer.insert(buffer.end_iter, word['word'], 'header')

				# show word type ("v.")
				buffer.insert(buffer.end_iter, ' ' + @@word_type_names[word['type']] + '.', 'word-type')

				# show definition(s) in a comma-separated list ("to speak, to talk")
				buffer.insert(buffer.end_iter, ' ' + definitions.collect { |definition| definition['word'] }.join(", ") + "\n", 'definition')
			end

			if result[:word_type] == WORD_TYPE_VERB
				# show verb table for infinitives
				if result[:verb_tense] == VERB_TENSE_INFINITIVE
					# for each tense (past perfect, etc.)
					@@tense_names.each_index { |tense|
						next if @@tense_show[tense] == false	# some tenses aren't so useful...

						# show tense name ("Past Perfect")
						buffer.insert(buffer.end_iter, "\n" + @@tense_names[tense] + "\n", 'tense')

						# for each person (yo, tu, el, ...)
						@@verb_person_names.each_index { |person|
							next if @@verb_person_show[person] == false		# skip "vosotros" based on user settings

							# show conjugation
							# attempt to load irregular conjugation (only irregulars are stored in the DB)
							irregular_conjugation = $db.get_first_row("SELECT * FROM spanish_verb_conjugation WHERE word_id=? AND tense=? AND person=?", result[:word_id], tense, person)
							if irregular_conjugation != nil
								buffer.insert(buffer.end_iter, irregular_conjugation['conjugation'] || '', 'indent', 'irregular-conjugation');
							else
								# this verb is regular in this tense
								conjugation = self.class.conjugate(word['word'], tense, person)
								next if conjugation.nil?	# some tenses (eg. gerund) don't have all persons
								buffer.insert(buffer.end_iter, conjugation, 'indent', 'regular-conjugation');
							end
							buffer.insert(buffer.end_iter, " (" + @@verb_person_names[person] + ")", 'person') if @@verb_person_names[person] != ''
							buffer.insert(buffer.end_iter, "\n");
						}
					}
				else
					# not an infinitive
					# show the tense ("past simple tense tú form")
					buffer.insert(buffer.end_iter, '↳ ' + @@tense_names[result[:verb_tense]] + ' tense', 'word-details')
					buffer.insert(buffer.end_iter, ' ' + @@verb_person_names[result[:verb_person]] + ' form', 'word-details') if @@verb_person_names[ result[:verb_person] ] != ''
					buffer.insert(buffer.end_iter, "\n")
				end
			end

			last_word_id = result[:word_id]
		}
	end

	###################################################################
	# render_results_to_notification() - take an array of results (as created
	#   by search() above) and return text suitable for a notification balloon
	###################################################################
	def render_results_to_notification(results)
		last_word_id = nil

		return results.collect { |result|
			txt = ''
			if last_word_id != result[:word_id]
				# blank line *between* different words (not before the first)
				txt += "\n\n" unless last_word_id.nil?

				# load up the word details and definitions
				word = $db.get_first_row("SELECT * FROM spanish_word WHERE id=?", result[:word_id])
				definitions = $db.execute("SELECT english_word.* FROM spanish_english LEFT JOIN english_word ON (english_word.id=spanish_english.english_word_id) WHERE spanish_english.spanish_word_id=?", result[:word_id])

				# show base word ("hablar")
				txt += '<b>' + word['word'] + '</b>'

				# show word type ("v.")
				txt += ' <i>' + @@word_type_names[word['type']] + '</i>.'

				# show definition(s) in a comma-separated list ("to speak, to talk")
				txt += ' ' + definitions.collect { |definition| definition['word'] }.join(", ")
			end

			# show tense/person for verbs
			if result[:word_type] == WORD_TYPE_VERB
				txt += "\n  ↳ "
				if result[:verb_tense] == VERB_TENSE_INFINITIVE
					txt += '<i>' + 'infinitive' + '</i>'
				else
					txt += '<i>' + @@tense_names[result[:verb_tense]] + ' tense' + '</i>'
					txt += ', <i>' + @@verb_person_names[result[:verb_person]] + ' form' + '</i>' unless @@verb_person_names[result[:verb_person]].empty?
				end
			end
			last_word_id = result[:word_id]

			txt	# return txt to collect()
		}.join
	end
end
